﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using MVCControlsToolkit.Core;
using System.Web.Http.ModelBinding;
using System.Runtime.Serialization;
using System.Web.Http;
using System.Net.Http;
namespace MVCControlsToolkit.Controller
{
    
    [DataContract]
    public class ApiServerError
    {
        [DataMember]
        public string id { get; set; }
        [DataMember]
        public string name { get; set; }
        [DataMember]
        public string[] errors { get; set; }
    }
    [DataContract]
    public class ApiKeyInfos<T>
    {
        [DataMember]
        public string destinationExpression { get; set; }
        [DataMember]
        public T[] keys { get; set; }
    }
    [DataContract]
    public class ApiServerErrors
    {
        [DataMember]
        public ApiServerError[] errors { get; set; }
        [DataMember]
        public ChangeInfo[] modelChanges { get; set; }
        private string[] errorList(ModelState entry)
        {
            List<string> res = new List<string>();
            foreach (var x in entry.Errors)
            {
                if (x.ErrorMessage != null) res.Add(x.ErrorMessage);
                else if (x.Exception != null) res.Add(x.Exception.Message);
            }
            if (res.Count == 0) res.Add("");
            return res.ToArray();
        }
        public void computeErrors(ModelStateDictionary origin, string prefix = null)
        {
            List<ApiServerError> res = new List<ApiServerError>();

            foreach (KeyValuePair<string, ModelState> pair in origin)
            {
                if (pair.Value.Errors.Count > 0 && (prefix == null || pair.Key.StartsWith(prefix)))
                {
                    res.Add(new ApiServerError
                    {
                        name = pair.Key,
                        id = BasicHtmlHelper.IdFromName(pair.Key),
                        errors = errorList(pair.Value)
                    });
                }
            }

            errors = res.ToArray();
        }
        public ApiServerErrors(ModelStateDictionary origin, ChangeInfo[] modelChanges, string prefix = null)
            
        {
            this.modelChanges = modelChanges;
            computeErrors(origin, prefix);
            
        }
        public ApiServerErrors(ModelStateDictionary origin, string prefix = null)
        {
            computeErrors(origin, prefix);
        }
        public HttpResponseMessage Wrap(HttpRequestMessage request)
        {
            if (errors.Length > 0) return request.CreateResponse(System.Net.HttpStatusCode.BadRequest, this);
            else return request.CreateResponse(System.Net.HttpStatusCode.OK, this);
        }

    }
    [DataContract]
    public class ApiServerErrors<T> : ApiServerErrors
    {
        [DataMember]
        public ApiKeyInfos<T>[] insertedKeys{get; set;}
        public new HttpResponseMessage Wrap(HttpRequestMessage request)
        {
            if (errors.Length > 0) return request.CreateResponse(System.Net.HttpStatusCode.BadRequest, this);
            else return request.CreateResponse(System.Net.HttpStatusCode.OK, this);
        }
           
        public ApiServerErrors(ModelStateDictionary origin, ApiKeyInfos<T>[] insertedKeys, string prefix = null, ChangeInfo[] modelChanges=null):base(origin, modelChanges, prefix)
        {
            this.insertedKeys=insertedKeys;
            this.modelChanges = modelChanges;
        }
        
        public ApiServerErrors(ModelStateDictionary origin, T[] keys, string destinationExpression = null, string prefix = null, ChangeInfo[] modelChanges = null)
            : base(origin, modelChanges, prefix)
        {
            this.insertedKeys = new ApiKeyInfos<T>[] { new ApiKeyInfos<T> { keys = keys, destinationExpression = destinationExpression } };
            this.modelChanges = modelChanges;
        }
    }
    public static class ApiServerErrorHelper
    {
        public static ApiServerErrors GetErrors(ModelStateDictionary modelState)
        {
            if (modelState == null) throw new ArgumentNullException("modelState");
            return new ApiServerErrors(modelState, null);
        }

        public static ApiServerErrors GetErrors<M, T>(M viewModel, Expression<Func<M, T>> expression, ModelStateDictionary modelState)
        {
            if (viewModel == null) throw new ArgumentNullException("viewModel");
            if (modelState == null) throw new ArgumentNullException("modelState");
            if (expression == null) throw new ArgumentNullException("expression");

            return new ApiServerErrors(modelState, System.Web.Mvc.ExpressionHelper.GetExpressionText(expression));
        }
    }
}
