﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using MVCControlsToolkit.Core;
using System.Web;
using System.Web.Http;
using System.Net.Http;

namespace MVCControlsToolkit.Linq
{
    public class HttpSafeQuery<T> : SafeQuery<T>
    {
        internal HttpRequestMessage request;
        
        public HttpSafeQuery(HttpRequestMessage request, IQueryable<T> query, SafeQueryProvider provider):base(query, provider)
        {
            this.request = request;
            (provider as HttpSafeQueryProvider).request = request;
        }
        public HttpSafeQuery(HttpRequestMessage request, IQueryable<T> query)
            : base(query, new HttpSafeQueryProvider(query.Provider, query.Expression))
        {
            this.request = request;
            (this.Provider as HttpSafeQueryProvider).request = this.request;
        }
    }

    public class HttpSafeQueryProvider : SafeQueryProvider
    {
        internal HttpRequestMessage request;
        internal HttpSafeQueryProvider(IQueryProvider provider, Expression initialExpression):base(provider, initialExpression)
        {
        }
        public override IQueryable<TElement> CreateQuery<TElement>(System.Linq.Expressions.Expression expression)
        {
            CheckExpression(expression);
            IQueryable<TElement> query = _inner.CreateQuery<TElement>(expression);
            SafeQueryProvider prov = new HttpSafeQueryProvider(query.Provider, expression);
            return new HttpSafeQuery<TElement>(request, query, prov);
        }

        public override IQueryable CreateQuery(System.Linq.Expressions.Expression expression)
        {
            CheckExpression(expression);
            IQueryable query = _inner.CreateQuery(expression);
            SafeQueryProvider prov = new HttpSafeQueryProvider(query.Provider, expression);
            Type elementType = TypeSystem.GetElementType(expression.Type);
            try
            {
                return (IQueryable)Activator.CreateInstance(typeof(HttpSafeQuery<>).MakeGenericType(elementType), new object[] {request, query, prov });
            }
            catch (System.Reflection.TargetInvocationException tie)
            {
                throw tie.InnerException;
            }
        }
        protected override void CheckExpression(Expression expression)
        {
            try
            {
                base.CheckExpression(expression);
            }
            catch (ForbiddenException ex)
            { 
                throw new HttpResponseException(request.CreateResponse(System.Net.HttpStatusCode.Forbidden, ex.Message));
            }
        }
    }
}
